#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "xmutil.h"
#include "prim.h"

static void glinit(Widget w, void *cls);
static void gldraw(Widget w, void *cls);
static void glreshape(Widget w, int x, int y, void *cls);
static void glkeyb(Widget w, int key, int press, void *cls);
static void glmouse(Widget w, int bn, int st, int x, int y, void *cls);
static void glmotion(Widget w, int x, int y, void *cls);
static int init_gui(void);
static void cbox_handler(Widget w, void *cls, void *calldata);
static void slider_handler(Widget w, void *cls, void *calldata);
static void create_menu(void);
static void file_menu_handler(Widget lst, void *cls, void *calldata);
static void help_menu_handler(Widget lst, void *cls, void *calldata);


XtAppContext app;
Widget app_shell;

static const char *glrend, *glvendor, *glver;

static Widget win, lb_status, viewport;
static Widget slider_hrot, slider_vrot, slider_dist;
static float cam_theta, cam_phi = 25, cam_dist = 8;
static int wire, shade = 1, gouraud = 1;

static int sphere, box;

int main(int argc, char **argv)
{
	if(!(app_shell = XtVaOpenApplication(&app, "glwtest", 0, 0, &argc, argv,
					0, sessionShellWidgetClass, NULL))) {
		fprintf(stderr, "failed to initialize ui\n");
		return 1;
	}
	XtVaSetValues(app_shell, XmNtitle, "Motif GL widget test", NULL);
	XtVaSetValues(app_shell, XmNallowShellResize, True, NULL);

	win = XmCreateMainWindow(app_shell, "mainwin", 0, 0);
	XtManageChild(win);

	if(init_gui() == -1) {
		return 1;
	}

	XtRealizeWidget(app_shell);
	XtAppMainLoop(app);
	return 0;
}

static void glinit(Widget w, void *cls)
{
	glvendor = (const char*)glGetString(GL_VENDOR);
	glrend = (const char*)glGetString(GL_RENDERER);
	glver = (const char*)glGetString(GL_VERSION);

	glClearColor(0.25, 0.25, 0.25, 1);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	sphere = glGenLists(1);
	glNewList(sphere, GL_COMPILE);
	draw_sphere(1, 24, 12);
	glEndList();

	box = glGenLists(1);
	glNewList(box, GL_COMPILE);
	draw_box(6, 0.2, 6);
	glEndList();
}

static void set_material(float r, float g, float b, float spec, float shin)
{
	float col[4] = {0, 0, 0, 1};
	col[0] = r; col[1] = g; col[2] = b;
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, col);
	col[0] = col[1] = col[2] = spec;
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, col);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shin);
}

static void gldraw(Widget w, void *cls)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0, 0, -cam_dist);
	glRotatef(cam_phi, 1, 0, 0);
	glRotatef(cam_theta, 0, 1, 0);

	glPushMatrix();
	glTranslatef(0, -0.1, 0);
	set_material(0.2, 0.4, 0.3, 0, 0);
	glCallList(box);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(0, 1, 0);
	set_material(0.2, 0.3, 1.0, 0.9, 60.0);
	glCallList(sphere);
	glPopMatrix();

	xm_gl_swap_buffers(w);
}

static void glreshape(Widget w, int x, int y, void *cls)
{
	glViewport(0, 0, x, y);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50.0f, (float)x / (float)y, 0.5, 500.0);
}

static void glkeyb(Widget w, int key, int press, void *cls)
{
	if(!press) return;

	switch(key) {
	case 27:
		exit(0);

	default:
		break;
	}
}

static unsigned int bnstate;
static int prev_mx, prev_my;

static void glmouse(Widget w, int bn, int st, int x, int y, void *cls)
{
	if(st) {
		bnstate |= 1 << bn;
	} else {
		bnstate &= ~(1 << bn);
	}
	prev_mx = x;
	prev_my = y;
}

static void glmotion(Widget w, int x, int y, void *cls)
{
	int dx = x - prev_mx;
	int dy = y - prev_my;
	prev_mx = x;
	prev_my = y;

	if((dx | dy) == 0) return;

	if(bnstate & 1) {
		cam_theta = fmod(cam_theta + dx * 0.5f, 360.0f);
		if(cam_theta < 0) cam_theta += 360.0f;
		cam_phi += dy * 0.5f;
		if(cam_phi < -90) cam_phi = -90;
		if(cam_phi > 90) cam_phi = 90;
		xm_set_sliderf_value(slider_hrot, cam_theta > 180.0f ? cam_theta - 360 : cam_theta);
		xm_set_sliderf_value(slider_vrot, cam_phi);
		xm_gl_redraw(w);
	}
	if(bnstate & 4) {
		cam_dist += dy * 0.1f;
		if(cam_dist < 1) cam_dist = 1;
		if(cam_dist > 50.0f) cam_dist = 50.0f;
		xm_set_sliderf_value(slider_dist, cam_dist);
		xm_gl_redraw(w);
	}
}

static int init_gui(void)
{
	Widget main;
	Widget frm_view, frm_rend;
	Widget vbox, vbox_ui;

	create_menu();

	main = XtVaCreateWidget("main", xmPanedWindowWidgetClass, win,
			XmNorientation, XmHORIZONTAL, NULL);

	vbox_ui = xm_rowcol(main, XmVERTICAL);

	viewport = xm_gl(main, 640, 480, XM_GL_DOUBLE | XM_GL_DEPTH);
	xm_gl_init_func(viewport, glinit, 0);
	xm_gl_draw_func(viewport, gldraw, 0);
	xm_gl_reshape_func(viewport, glreshape, 0);
	xm_gl_keyboard_func(viewport, glkeyb, 0);
	xm_gl_mouse_func(viewport, glmouse, 0);
	xm_gl_motion_func(viewport, glmotion, 0);

	frm_rend = xm_frame(vbox_ui, "Render options");
	vbox = xm_rowcol(frm_rend, XmVERTICAL);
	xm_checkbox(vbox, "wireframe", wire, cbox_handler, 0);
	xm_checkbox(vbox, "lighting", shade, cbox_handler, (void*)1);
	xm_checkbox(vbox, "gouraud", gouraud, cbox_handler, (void*)2);

	frm_view = xm_frame(vbox_ui, "View");
	vbox = xm_rowcol(frm_view, XmVERTICAL);
	slider_hrot = xm_sliderf(vbox, "horizontal", cam_theta, -180, 180, slider_handler, 0);
	slider_vrot = xm_sliderf(vbox, "vertical", cam_phi, -90, 90, slider_handler, (void*)1);
	slider_dist = xm_sliderf(vbox, "zoom", cam_dist, 1, 50, slider_handler, (void*)2);

	XtManageChild(main);

	lb_status = xm_label(win, "");
	XtVaSetValues(win, XmNmessageWindow, lb_status, NULL);
	return 0;
}

static void cbox_handler(Widget w, void *cls, void *calldata)
{
	int id = (int)cls;
	XmToggleButtonCallbackStruct *cdata = calldata;

	switch(id) {
	case 0:
		wire = cdata->set;
		glPolygonMode(GL_FRONT_AND_BACK, wire ? GL_LINE : GL_FILL);
		xm_gl_redraw(viewport);
		break;

	case 1:
		shade = cdata->set;
		if(shade) {
			glEnable(GL_LIGHTING);
		} else {
			glDisable(GL_LIGHTING);
		}
		xm_gl_redraw(viewport);
		break;

	case 2:
		gouraud = cdata->set;
		glShadeModel(gouraud ? GL_SMOOTH : GL_FLAT);
		xm_gl_redraw(viewport);
		break;

	default:
		break;
	}
}

static void slider_handler(Widget w, void *cls, void *calldata)
{
	int id = (int)cls;
	float val = xm_get_sliderf_value(w);

	switch(id) {
	case 0:
		cam_theta = val;
		xm_gl_redraw(viewport);
		break;

	case 1:
		cam_phi = val;
		xm_gl_redraw(viewport);
		break;

	case 2:
		cam_dist = val;
		xm_gl_redraw(viewport);
		break;

	default:
		break;
	}
}

static void create_menu(void)
{
	XmString sfile, shelp, snew, sopen, ssave, ssave_key, squit, sabout, squit_key;
	Widget menubar;

	sfile = XmStringCreateSimple("File");
	shelp = XmStringCreateSimple("Help");
	menubar = XmVaCreateSimpleMenuBar(win, "menubar", XmVaCASCADEBUTTON, sfile, 'F',
			XmVaCASCADEBUTTON, shelp, 'H', NULL);
	XtManageChild(menubar);
	XmStringFree(sfile);
	XmStringFree(shelp);

	snew = XmStringCreateSimple("New");
	sopen = XmStringCreateSimple("Open");
	ssave = XmStringCreateSimple("Save");
	ssave_key = XmStringCreateSimple("Ctrl-S");
	squit = XmStringCreateSimple("Quit");
	squit_key = XmStringCreateSimple("Ctrl-Q");
	XmVaCreateSimplePulldownMenu(menubar, "filemenu", 0, file_menu_handler,
			XmVaPUSHBUTTON, snew, 'N', NULL, NULL,
			XmVaPUSHBUTTON, sopen, 'O', NULL, NULL,
			XmVaPUSHBUTTON, ssave, 'S', "Ctrl<Key>s", ssave_key,
			XmVaPUSHBUTTON, squit, 'Q', "Ctrl<Key>q", squit_key,
			NULL);
	XmStringFree(ssave);
	XmStringFree(ssave_key);
	XmStringFree(squit);
	XmStringFree(squit_key);

	sabout = XmStringCreateSimple("About");
	XmVaCreateSimplePulldownMenu(menubar, "helpmenu", 1, help_menu_handler,
			XmVaPUSHBUTTON, sabout, 'A', NULL, NULL, NULL);
	XmStringFree(sabout);
}

static void file_menu_handler(Widget lst, void *cls, void *calldata)
{
	int item = (int)cls;
	switch(item) {
	case 0:	/* new */
	case 1:	/* open */
	case 2:	/* save */
		break;
	case 3: /* quit */
		exit(0);
	}
}


static void help_menu_handler(Widget lst, void *cls, void *calldata)
{
	static const char *about_fmt = "GLw widget test\n\n"
		"%s\n\n"
		"OpenGL information:\n"
		" Vendor: %s\n"
		" Renderer: %s\n"
		" Version: %s\n"
		"\n";
	char *textbuf;
	int len;

#ifdef XmVERSION_STRING
	const char *motifver = XmVERSION_STRING;
#else
	char motifver[64];
	sprintf(motifver, "Motif %d.%d", XmVERSION, XmREVISION);
#endif

	if(!glrend) {
		glrend = glver = glvendor = "???";
	}
	len = strlen(about_fmt) + strlen(motifver) + strlen(glrend) +
		strlen(glvendor) + strlen(glver);
	if(!(textbuf = malloc(len + 1))) {
		return;
	}
	sprintf(textbuf, about_fmt, motifver, glvendor, glrend, glver);

	messagebox(XmDIALOG_INFORMATION, "About glwtest", textbuf);
	free(textbuf);
}

static void set_status(const char *s)
{
	XmString xs = XmStringCreateSimple((char*)s);
	XtVaSetValues(lb_status, XmNlabelString, xs, NULL);
	XmStringFree(xs);
}
