#include <math.h>
#include <GL/gl.h>
#include "prim.h"

static void sphvertex(float u, float v, float r)
{
	float theta = u * (M_PI * 2.0);
	float phi = v * M_PI;
	float x = sin(theta) * sin(phi);
	float y = cos(phi);
	float z = cos(theta) * sin(phi);

	glTexCoord2f(u, v);
	glNormal3f(x, y, z);
	glVertex3f(x * r, y * r, z * r);
}

void draw_sphere(float rad, int usub, int vsub)
{
	int i, j;
	float u, v, du, dv;

	du = 1.0f / (float)usub;
	dv = 1.0f / (float)vsub;

	glBegin(GL_QUADS);
	u = 0.0f;
	for(i=0; i<usub; i++) {
		v = 0.0f;
		for(j=0; j<vsub; j++) {
			sphvertex(u, v + dv, rad);
			sphvertex(u + du, v + dv, rad);
			sphvertex(u + du, v, rad);
			sphvertex(u, v, rad);
			v += dv;
		}
		u += du;
	}
	glEnd();
}

void draw_box(float xsz, float ysz, float zsz)
{
	xsz /= 2.0f;
	ysz /= 2.0f;
	zsz /= 2.0f;

	glBegin(GL_QUADS);
	/* +Z */
	glNormal3f(0, 0, 1);
	glTexCoord2f(0, 1); glVertex3f(-xsz, -ysz, zsz);
	glTexCoord2f(1, 1); glVertex3f(xsz, -ysz, zsz);
	glTexCoord2f(1, 0); glVertex3f(xsz, ysz, zsz);
	glTexCoord2f(0, 0); glVertex3f(-xsz, ysz, zsz);
	/* +X */
	glNormal3f(1, 0, 0);
	glTexCoord2f(0, 1); glVertex3f(xsz, -ysz, zsz);
	glTexCoord2f(1, 1); glVertex3f(xsz, -ysz, -zsz);
	glTexCoord2f(1, 0); glVertex3f(xsz, ysz, -zsz);
	glTexCoord2f(0, 0); glVertex3f(xsz, ysz, zsz);
	/* -Z */
	glNormal3f(0, 0, -1);
	glTexCoord2f(0, 1); glVertex3f(xsz, -ysz, -zsz);
	glTexCoord2f(1, 1); glVertex3f(-xsz, -ysz, -zsz);
	glTexCoord2f(1, 0); glVertex3f(-xsz, ysz, -zsz);
	glTexCoord2f(0, 0); glVertex3f(xsz, ysz, -zsz);
	/* -X */
	glNormal3f(-1, 0, 0);
	glTexCoord2f(0, 1); glVertex3f(-xsz, -ysz, -zsz);
	glTexCoord2f(1, 1); glVertex3f(-xsz, -ysz, zsz);
	glTexCoord2f(1, 0); glVertex3f(-xsz, ysz, zsz);
	glTexCoord2f(0, 0); glVertex3f(-xsz, ysz, -zsz);
	/* +Y */
	glNormal3f(0, 1, 0);
	glTexCoord2f(0, 1); glVertex3f(-xsz, ysz, zsz);
	glTexCoord2f(1, 1);	glVertex3f(xsz, ysz, zsz);
	glTexCoord2f(1, 0); glVertex3f(xsz, ysz, -zsz);
	glTexCoord2f(0, 0); glVertex3f(-xsz, ysz, -zsz);
	/* -Y */
	glNormal3f(0, -1, 0);
	glTexCoord2f(0, 1); glVertex3f(-xsz, -ysz, -zsz);
	glTexCoord2f(1, 1);	glVertex3f(xsz, -ysz, -zsz);
	glTexCoord2f(1, 0); glVertex3f(xsz, -ysz, zsz);
	glTexCoord2f(0, 0); glVertex3f(-xsz, -ysz, zsz);
	glEnd();
}
